﻿Namespace ViewModel
    ''' <summary>
    ''' Diese Klasse Implementiert das ICommand Interface, so muss man nicht in jeder Klasse eines ViewModel alles selbst implementieren.
    ''' <example>
    ''' <para>Implementierung könnte folgendermaßen aussehen:</para>
    ''' <code>
    ''' MyCommand = New RelayCommand(AddressOf MyCommand_Execute, AddressOf MyCommand_CanExecute)
    ''' </code>
    ''' </example>
    ''' </summary>
    Public Class RelayCommand : Implements ICommand

#Region " Fields                        "

        Private ReadOnly _execute As Action(Of Object)
        Private ReadOnly _canExecute As Predicate(Of Object)

#End Region

#Region " Constructors"

        ''' <summary>
        ''' Erstellt einen neuen Command welcher CanExecute nicht setzt, somit ist dieser Command dann immer Executeable.
        ''' </summary>
        ''' <param name="execute">The execution logic.</param>
        Public Sub New(execute As Action(Of Object))
            Me.New(execute, Nothing)
        End Sub

        ''' <summary>
        ''' Erstellt einen neuen Command welcher sowohl die Execute als auch die CanExecute Logik beinhaltet.
        ''' </summary>
        ''' <param name="execute">Die Logik für Execute.</param>
        ''' <param name="canExecute">Die Logik für CanExecute.</param>
        Public Sub New(execute As Action(Of Object), canExecute As Predicate(Of Object))
            If execute Is Nothing Then
                Throw New ArgumentNullException("execute")
            End If

            _execute = execute
            _canExecute = canExecute
        End Sub

#End Region

#Region " ICommand Members              "

        ''' <summary>
        ''' Setzt die CanExecute-Methode des ICommand-Interfaces auf True oder False
        ''' </summary>
        ''' <param name="parameter"></param>
        ''' <returns>Gibt zurück ob die Aktion ausgeführt werden kann oder nicht</returns>
        Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
            Return _canExecute Is Nothing OrElse _canExecute(parameter)
        End Function

        ''' <summary>
        ''' Event welches geworfen wird wenn die Property CanExecuteChanged sich ändert.
        ''' </summary>
        Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
            AddHandler(value As EventHandler)
                If _canExecute IsNot Nothing Then
                    AddHandler CommandManager.RequerySuggested, value
                End If
            End AddHandler
            RemoveHandler(value As EventHandler)
                If _canExecute IsNot Nothing Then
                    RemoveHandler CommandManager.RequerySuggested, value
                End If
            End RemoveHandler
            RaiseEvent(sender As Object, e As EventArgs)
            End RaiseEvent
        End Event

        ''' <summary>
        ''' Führt die Prozedur Execute des ICommand.Execute aus
        ''' </summary>
        ''' <param name="parameter"></param>
        ''' <remarks></remarks>
        Public Sub Execute(parameter As Object) Implements ICommand.Execute
            _execute(parameter)
        End Sub

#End Region

    End Class
End Namespace
